/*******************************************************************************
 *   Webster II - The Mini Webserver
 *
 *   Code to manipulate static network settings.
 *   Functionality extracted from Netburner example program ChangeIP.
 *
 *   Written by: Craig A. Lindley
 *   Version: 1.0
 *   Last Update: 07/30/2008
 ******************************************************************************/

/*------------------------------------------------------------------------

 INTRODUCTION TO CONFIGURATION RECORDS AND INTERFACE BLOCKS
 - The NetBurner device contains runtime and stored (Flash) system
   configuration parameters. Interface Blocks are used at runtime,
   and Configuration Records are stored in Flash. At boot time,
   data from the Configuration Records are copied to Interface
   Blocks.

 - There are 3 Configuration Records as defined in netinterfaces.h:
     #define CONFIG_IF_ID_ETHERNET  (0)
     #define CONFIG_IF_ID_WIFI      (1)
     #define CONFIG_IF_ID_ETHERNET2 (2)
     
   The order of these configuration records is fixed, regardless of
   the Interface Block number. For example, if you have only a WiFi
   interface, you still use Configuration Record 1. 
   
 - Configuration Records are numbered 0, 1 and 2. Interface Blocks
   are numbered 1, 2, 3. There is no correlation between this 
   numbering. Interface Block 3 could reference Configuration Record
   1, depending on the order of interface registration calls by your
   application.

------------------------------------------------------------------------*/

#include <stdio.h>
#include <startnet.h>
#include <string.h>
#include <netinterface.h>

// Change the runtime network settings. 
void ChangeRTNetworkSettings(int InterfaceNumber, IPADDR IpAddr, IPADDR IpMask, IPADDR IpGate, IPADDR IpDNS) {
	
	iprintf("\r\nChanging runtime settings for interface %d:\r\n", InterfaceNumber);
	
	// Get block pointer
	InterfaceBlock *ib = GetInterFaceBlock(InterfaceNumber);
	
	// Poke in the new values
	ib->netIP = IpAddr;
	ib->netIpMask = IpMask;
	ib->netIpGate = IpGate;
	ib->netDNS = IpDNS;
}

// Change the network settings stored in flash
void ChangeFlashNetworkSettings(int RecordNumber, IPADDR IpAddr, IPADDR IpMask, IPADDR IpGate, IPADDR IpDNS) {
	
	iprintf("\r\nChanging Flash settings for record %d:\r\n", RecordNumber);
	
	// Get pointer to Configuration Record
	ConfigRecord *cr = RawGetConfig(RecordNumber);
	
	// create new config record and copy data
	ConfigRecord NewRec;
	memcpy(&NewRec, cr, sizeof(NewRec));
	
	// Change parameters
	NewRec.ip_Addr = IpAddr;
	NewRec.ip_Mask = IpMask;
	NewRec.ip_GateWay = IpGate;
	NewRec.ip_DNS_server = IpDNS;
	
	// Write new values to Flash system configuration sector
	UpdateConfigRecord_Num(&NewRec, RecordNumber);
}

